@@ -2,13 +2,24 @@ use crate::{crossdev, InodeFilter, WalkOptions, WalkResult};
 use anyhow::Result;
 use colored::{Color, Colorize};
 use filesize::PathExt;
-use std::{borrow::Cow, io, path::Path};
+use std::time::Duration;
+use std::{
+    borrow::Cow,
+    io,
+    path::Path,
+    sync::{
+        atomic::{AtomicU64, Ordering},
+        Arc,
+    },
+    thread,
+};
 
 /// Aggregate the given `paths` and write information about them to `out` in a human-readable format.
 /// If `compute_total` is set, it will write an additional line with the total size across all given `paths`.
 /// If `sort_by_size_in_bytes` is set, we will sort all sizes (ascending) before outputting them.
 pub fn aggregate(
     mut out: impl io::Write,
+    err: Option<impl io::Write + Send + 'static>,
     walk_options: WalkOptions,
     compute_total: bool,
     sort_by_size_in_bytes: bool,
@@ -24,6 +35,26 @@ pub fn aggregate(
     let mut aggregates = Vec::new();
     let mut inodes = InodeFilter::default();
     let paths: Vec<_> = paths.into_iter().collect();
+    let shared_count = Arc::new(AtomicU64::new(0));
+
+    if let Some(mut err) = err {
+        thread::spawn({
+            let shared_count = Arc::clone(&shared_count);
+            move || {
+                thread::sleep(Duration::from_secs(1));
+                loop {
+                    thread::sleep(Duration::from_millis(100));
+                    write!(
+                        err,
+                        "Enumerating {} entries\r",
+                        shared_count.load(Ordering::Acquire)
+                    )
+                    .ok();
+                }
+            }
+        });
+    }
+
     for path in paths.into_iter() {
         num_roots += 1;
         let mut num_bytes = 0u128;
@@ -31,6 +62,7 @@ pub fn aggregate(
         let device_id = crossdev::init(path.as_ref())?;
         for entry in walk_options.iter_from_path(path.as_ref()) {
             stats.entries_traversed += 1;
+            shared_count.fetch_add(1, Ordering::Relaxed);
             match entry {
                 Ok(entry) => {
                     let file_size = match entry.client_state {
@@ -54,7 +86,7 @@ pub fn aggregate(
                             num_errors += 1;
                             0
                         }
-                        None => unreachable!("must have populated client state for metadata"),
+                        None => 0, // ignore directory
                     } as u128;
                     stats.largest_file_in_bytes = stats.largest_file_in_bytes.max(file_size);
                     stats.smallest_file_in_bytes = stats.smallest_file_in_bytes.min(file_size);
